;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;ToggleReverseTurn
        (ChordAnnotation "ToggleReverseTurn" "\\reverseturn"    ToggleReverseTurn::params    LG-ReverseTurn)
