;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
  ;;;;;;;;;;;;;;
(d-DirectivePut-movementcontrol-override "MvntComment" (logior DENEMO_OVERRIDE_GRAPHIC DENEMO_OVERRIDE_EDITOR))
(d-DirectivePut-movementcontrol-display "MvntComment" (_ "Click to edit comment"))

