;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;QuickLilyPondExport
(d-ExportMUDELA (let ((f (d-Open "query=filename"))) 
	 (substring f 0 (- (string-length f) 7))))