;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
 ;SlurNeutral:
(d-DirectivePut-standalone-display "SlurDirection" "Auto Slur Direction")
(d-MoveCursorLeft)
(d-DirectivePut-standalone-postfix "SlurDirection" " \\slurNeutral" )
(d-DirectivePut-standalone-minpixels "SlurDirection" 75)
(d-MoveCursorRight)
(d-RefreshDisplay)