;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;ToggleUpPrall
        (ChordAnnotation "ToggleUpPrall" "\\upprall"    ToggleUpPrall::params    LG-UpPrall)
;;(d-DirectivePut-chord-gy "UpPrall" -100)
