;;; Warning!!! This file is derived from those in actions/menus/... do not edit here


(ChordComparison::ChordComparison (list ChordComparison::Major7 ChordComparison::Minor7 ChordComparison::Dominant7))

