;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;Half Barline
(StandAloneDirectiveProto (cons "HalfBarline" "\\once \\override Staff . BarLine #'bar-size = #2 \\bar \"|\""))