;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;Tempo
(d-DirectivePut-chord-postfix "Tempo" "^\\markup \\italic \\bold {Allegro Assai}")
(d-DirectivePut-chord-display "Tempo" "Allegro Assai")
(d-RefreshDisplay)
;;;End of scheme script