;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;; Andante
(d-DirectivePut-chord-postfix "Tempo" "^\\markup \\italic \\bold {Andante}")
(d-DirectivePut-chord-display "Tempo" "Andante")
(d-RefreshDisplay)