;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;TinyStaffAllMovements
(ForAllMovements (string-append "(d-GoToPosition #f " (number->string (d-GetStaff)) " 1 1)(d-TinyStaff)"))
