;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;Trill                   
					(let ((tag "ToggleTrill"))
						(ChordAnnotation tag "\\trill"    ToggleTrill::params    LG-Trill)
					(d-PlayMidiNote 78  255 9 100))
	