;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;ToggleHarmonic
        (ChordAnnotation "ToggleHarmonic" "\\flageolet"    ToggleHarmonic::params    LG-Harmonic)
;(d-DirectivePut-chord-gy "Harmonic" 12)
;(d-DirectivePut-chord-gx "Harmonic" -2)
