;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
 ;;;;;;;;BeamingOn
(d-DirectivePut-standalone "BeamingOn")
(d-DirectivePut-standalone-postfix "BeamingOn" "\\autoBeamOn")
(d-DirectivePut-standalone-display "BeamingOn" "Automatic Beaming")
(d-DirectivePut-standalone-minpixels "BeamingOn" 10)
(d-RefreshDisplay)
