;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
 ;;; ParenthesizeChord
(d-DirectivePut-chord-prefix "Parenthesize" "\\parenthesize ")
(d-DirectivePut-chord-display "Parenthesize" "()")
(d-RefreshDisplay)