;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;LongFermata
        (ChordAnnotation "ToggleLongFermata" "\\longfermata"    ToggleLongFermata::params    LG-LongFermata)
