;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;; DeleteBeamingDirective
(d-DirectiveDelete-chord "Beam")
(d-DirectiveDelete-chord "BeamLeft")
(d-DirectiveDelete-chord "BeamRight")