;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
(d-DirectivePut-score-postfix  "TransposeScore" "\\transpose c c' ")
(d-DirectivePut-score-display  "TransposeScorePrint" (_ "Print out 8va up"))
(d-DirectivePut-score-override  "TransposeScorePrint" DENEMO_OVERRIDE_GRAPHIC)

(d-RefreshDisplay)
