;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;ToggleStaccatissimo
        (ChordAnnotation "ToggleStaccatissimo" "\\staccatissimo"    ToggleStaccatissimo::params    LG-Staccatissimo)
