;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
 ;;;StaffOctaveUp
(d-DirectivePut-staff-prefix "StaffOctaveUp" "\\transpose c c'")
(d-DirectivePut-staff-graphic "StaffOctaveUp" "StaffOctaveUp")
(d-RefreshDisplay)