;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;; Allegro
(d-DirectivePut-chord-postfix "Tempo" "^\\markup \\italic \\bold {Allegro}")
(d-DirectivePut-chord-display "Tempo" "Allegro")
(d-RefreshDisplay)