;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;HideMovement
;delete obsolete directives
(d-DirectiveDelete-movementcontrol "HideMovement")
(d-DirectiveDelete-layout "HideMovement")
(d-DirectiveDelete-header "HideMovement")
(d-InfoDialog "Use the Score Layout view to create a typeset with movements omitted.\nUse the \"X\" button on the movements to be dropped.\nSee the View Menu, Score Layout")
