;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;NoteNamesMove
(d-AddKeybinding "MoveToA" "a")
(d-AddKeybinding "MoveToB" "b")
(d-AddKeybinding "MoveToC" "c")
(d-AddKeybinding "MoveToD" "d")
(d-AddKeybinding "MoveToE" "e")
(d-AddKeybinding "MoveToF" "f")
(d-AddKeybinding "MoveToG" "g")