;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;ScoreCopyright
		(SetScoreHeaderField "copyright" #f #f)
		(DenemoPrintAllHeaders)
		(d-SetSaved #f)
		(d-RefreshDisplay)
	