;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;; Presto
(d-DirectivePut-chord-postfix "Tempo" "^\\markup \\italic \\bold {Presto}")
(d-DirectivePut-chord-display "Tempo" "Presto")
(d-RefreshDisplay)