;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;ToggleSignumcongruentiae
        (ChordAnnotation "ToggleSignumcongruentiae" "\\signumcongruentiae"    ToggleSignumcongruentiae::params    LG-Signumcongruentiae)
