;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;in reality this is not shift but transpose. But there are too many functions with the name transpose already...
(SingleAndSelectionSwitcher (lambda () 
	(ShiftProto ANS::CalculateRealOctaveDown)))
