;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;;CloseLilyBlock
 (d-Directive-standalone "CloseLilyBlock")
 (d-DirectivePut-standalone-postfix "CloseLilyBlock" "}")
(d-DirectivePut-standalone-display "CloseLilyBlock" "}")
(d-DirectivePut-standalone-minpixels "CloseLilyBlock" 20)
(d-RefreshDisplay)