;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
 ;;;;;;;;;;CloseReopenRepeat
 (d-Directive-standalone "Repeat")
(d-DirectivePut-standalone-postfix "Repeat" "}\n\\repeat volta 2 {\n")
(d-DirectivePut-standalone-minpixels "Repeat" 25)
(d-DirectivePut-standalone-gx  "Repeat"  20)
(d-DirectivePut-standalone-gy  "Repeat"  -1)
(d-DirectivePut-standalone-graphic "Repeat" "RepeatCloseOpen")
(d-MoveCursorRight)
(d-RefreshDisplay)
