;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;;;;HideLilyPond
(d-DirectivePut-chord-prefix "HideLilyPond" "%{")
(d-DirectivePut-chord-postfix "HideLilyPond" "%}")
(d-DirectivePut-chord-display "HideLilyPond" (_ "hide"))
(d-DirectivePut-chord-override "HideLilyPond" DENEMO_OVERRIDE_AFFIX)
(d-SetSaved #f)