;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;AutomaticAccidentals
             (StandAloneDirectiveProto (cons "AutomaticAccidentals" "
        	#(set-accidental-style 'modern 'Score)        	
"))
