;;; Warning!!! This file is derived from those in actions/menus/... do not edit here

   	(set-cdr! wrap:Op0 (car wrap:Op0))
   	(set-cdr! wrap:Op1 (car wrap:Op1))
   	(set-cdr! wrap:Op2 (car wrap:Op2))
   	(set-cdr! wrap:Op3 (car wrap:Op3))
   	(set-cdr! wrap:Op4 (car wrap:Op4))
   	(set-cdr! wrap:Op5 (car wrap:Op5))
   	(set-cdr! wrap:Op6 (car wrap:Op6))
   	(set-cdr! wrap:Op7 (car wrap:Op7))
   	(set-cdr! wrap:Op8 (car wrap:Op8))
   	(set-cdr! wrap:Op9 (car wrap:Op9))
   	(Help::RemoveTag 'doublestroke)
        