;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;SmallFontEnd
     (StandAloneDirectiveProto (cons "SmallFontEnd" "%Restore normal note size
        	\\revert NoteHead #'font-size 
        	\\revert Accidental #'font-size
") #t "\ns")
		