;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;ToggleFine
        (ChordAnnotation "ToggleFine" "-\\markup \\italic \"fine\""    ToggleFine::params   #f "fine")
