;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
(if DenemoPref_immediateplayback
(d-SetPrefs "<immediateplayback>0</immediateplayback>")
(d-SetPrefs "<immediateplayback>1</immediateplayback>"))
