;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
(Doublestroke #f
(cons "Voice one preset" d-VoicePreset1) ;1
(cons "Voice two preset" d-VoicePreset2) ;2
(cons "Voice three preset" d-VoicePreset3) ;3
(cons "Voice four preset" d-VoicePreset4) ;4
(cons "Automatic voice or single voice" d-VoicePresetAutomatic) ;5
)