;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
 ;;;FirstTimeBar
 (d-Directive-standalone "Repeat")
(d-DirectivePut-standalone-minpixels "Repeat" 30)
(d-DirectivePut-standalone-postfix "Repeat" "
} \\alternative{{
	")
(d-DirectivePut-standalone-graphic "Repeat" "FirstTimeBar")
(d-DirectivePut-standalone-gx  "Repeat" 32)
(d-DirectivePut-standalone-gy  "Repeat" -34)
(d-MoveCursorRight)
(d-RefreshDisplay)