;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;MouseDefault
(d-AddKeybinding "NoOp" "PrsL-Shift")
(d-AddKeybinding "A" "a")
(d-AddKeybinding "B" "b")
(d-AddKeybinding "C" "c")
(d-AddKeybinding "D" "d")
(d-AddKeybinding "E" "e")
(d-AddKeybinding "F" "f")
(d-AddKeybinding "G" "g")

(d-AddKeybinding "Change0" "Shft+0")
(d-AddKeybinding "Change1" "Shft+1")
(d-AddKeybinding "Change2" "Shft+2")
(d-AddKeybinding "Change3" "Shft+3")
(d-AddKeybinding "Change4" "Shft+4")
(d-AddKeybinding "Change5" "Shft+5")
(d-AddKeybinding "Change6" "Shft+6")

(d-DirectiveDelete-score "Note0")
(d-DirectiveDelete-score "Note1")
(d-DirectiveDelete-score "Note2")
(d-DirectiveDelete-score "Note3")
(d-DirectiveDelete-score "Note4")
(d-DirectiveDelete-score "Note5")
(d-DirectiveDelete-score "Note6")

(d-DirectiveDelete-score "SharpenNext")
(d-DirectiveDelete-score "FlattenNext")

(d-DirectiveDelete-score "Dot - Double Dot - No Dot")

(d-DirectiveDelete-score "BeginSlur")
(d-DirectiveDelete-score "EndSlur")
(d-DirectiveDelete-score "Tie")
(d-DirectiveDelete-score "Triplets")
(d-DirectiveDelete-score "Dynamics")
(d-DirectiveDelete-score "Tempo")
(d-DirectiveDelete-score "StartCresc")
(d-DirectiveDelete-score "EndCresc")
(d-DirectiveDelete-score "StartDim")
(d-DirectiveDelete-score "EndDim")

