;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
(d-PushPosition)
(d-MoveToMovementBeginning)
(d-UnsetMark)
(let laststaff ()
(if (d-StaffDown)	 
	(laststaff)))
(d-GoToEnd)
(d-PopPosition)