;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
(d-DirectivePut-standalone-midibytes "EqualTemp" "0xf0 0x7f 0x7f 0x08 0x08   0x03 0x7f 0x7f    0x40 0x40 0x40 0x40 0x40 0x40 0x40 0x40 0x40 0x40 0x40 0x40   0xf7")
