;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;; StartCadenza
			(StandAloneDirectiveProto (cons "Cadenza" "\\cadenzaOn s8*0^\\markup \\bold {Cadenza ad lib.}" ))           
		