;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;NoteNamesDefault
(d-AddKeybinding "A" "a")
(d-AddKeybinding "B" "b")
(d-AddKeybinding "C" "c")
(d-AddKeybinding "D" "d")
(d-AddKeybinding "E" "e")
(d-AddKeybinding "F" "f")
(d-AddKeybinding "G" "g")