;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
(d-DirectiveDelete-staff "ContextGroupStaff")
(d-DirectiveDelete-staff "ContextPianoStaff")
(d-DirectiveDelete-staff "ContextChoirStaff")
(d-DirectiveDelete-staff "ContextGrandStaff")