;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;TogglePrallPrall
        (ChordAnnotation "TogglePrallPrall" "\\prallprall"    TogglePrallPrall::params    LG-PrallPrall)
