;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;Play from Cusor to End
(d-PushPosition)
(DenemoSetPlaybackStart)
(d-MoveToEnd)
(DenemoSetPlaybackEnd)
(d-PopPosition)
(d-Play)