;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
	
	(set-cdr! wrap:Op0 d-Set0)
	(set-cdr! wrap:Op1 d-Set1)
	(set-cdr! wrap:Op2 d-Set2)
	(set-cdr! wrap:Op3 d-Set3)
	(set-cdr! wrap:Op4 d-Set4)
	(set-cdr! wrap:Op5 d-Set5)
	(set-cdr! wrap:Op6 d-Set6)
	(set-cdr! wrap:Op7 d-Set7)
	(set-cdr! wrap:Op8 d-SetBreve)
	(set-cdr! wrap:Op9 d-SetLonga)
	(Help::RemoveTag 'doublestroke)