;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;Piece
          (MovementBookTitles::Do "Piece" "piece " "My Piece"  (_ "Give name for this piece of the current movement or blank out to delete: "))
        