;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
 ;;;;;;;;BeamingOff
(d-DirectivePut-standalone "BeamingOff")
(d-DirectivePut-standalone-postfix "BeamingOff" "\\autoBeamOff")
(d-DirectivePut-standalone-display "BeamingOff" (_ "No Beaming"))
(d-DirectivePut-standalone-minpixels "BeamingOff" 10)
(d-RefreshDisplay)
