;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;ToggleStaccato
        (ChordAnnotation "ToggleStaccato" "\\staccato"    ToggleStaccato::params    LG-Staccato)
