;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
(d-DirectivePut-chord-prefix "NoteHead" "\\once \\override NoteHead #'style = #'neomensural ")
(d-DirectivePut-chord-display "NoteHead" "nm")
(d-DirectivePut-chord-override "NoteHead" DENEMO_OVERRIDE_AFFIX)
(d-SetSaved #f)