;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;;OpenOctaveDown
 (d-Directive-standalone "OpenPrintTransposed")
 (d-DirectivePut-standalone-postfix "OpenPrintTransposed" "\\transpose c c,{")
(d-DirectivePut-standalone-display "OpenPrintTransposed" (_ "Down8va"))
(d-DirectivePut-standalone-minpixels "OpenPrintTransposed" 20)
(d-RefreshDisplay)