;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;VeryLongFermata
        (ChordAnnotation "ToggleVeryLongFermata" "\\verylongfermata"    ToggleVeryLongFermata::params    LG-VeryLongFermata)
