;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
 ;;;;; mean tone:
(d-DirectivePut-standalone-midibytes "QCMeanTone" "0xf0 0x7f 0x7f 0x08 0x08   0x03 0x7f 0x7f    0x40 40 57 74 50 67 44 61 37 54 71 47   0xf7")
