;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
 ;;;OpenSecondTimeBar
(d-Directive-standalone "Volta")
(d-DirectivePut-standalone-minpixels  "Volta" 50)
(d-DirectivePut-standalone-postfix "Volta" "
\\set Score.repeatCommands = #'((volta \"2\"))
")
(d-DirectivePut-standalone-gx  "Volta" 37)
(d-DirectivePut-standalone-gy  "Volta" -34)
(d-DirectivePut-standalone-graphic "Volta" "SecondTimeBar")
(d-MoveCursorRight)
(d-RefreshDisplay)
 