;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
(AttachDirective "layout" "postfix" "MovementNoteheads" "\t\\context {
\t\\Score		   
\t\\override NoteHead  #'style = #'harmonic-black }\n")