;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;ToggleShortFermata
        (ChordAnnotation "ToggleShortFermata" "\\shortfermata"    ToggleShortFermata::params    LG-ShortFermata)
