;;; Warning!!! This file is derived from those in actions/menus/... do not edit here

(define ChordComparison::Major (cons "Major" "c e g"))
(define ChordComparison::Minor (cons "Minor" "c ees g"))

(ChordComparison::ChordComparison (list ChordComparison::Major ChordComparison::Minor))

