;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;AdjustBassFigureHeights
(let ((tagbase "FBHeight")(number #f))
	(define offset #f)
	(set! number (d-GetUserInput (_ "Figured Bass Height")  (_ "Give number of groups: ") "2"))
	(if (and (string? number) (string->number number))
		(let ()
				(set! number (string->number number))
				(let loop ((count 1))
					(define tag (string-append tagbase (number->string count)))
					(set! offset (d-GetUserInput (_ "Figured Bass Height") (_ "Give height adjustment (unit = staff space): ") "1.0"))
					(if (and (string? offset) (string->number offset))
						(begin
							(d-DirectivePut-note-prefix tag (string-append "\\once \\override Staff.BassFigureAlignmentPositioning #'Y-offset = #'" offset " "))
							(d-DirectivePut-note-override tag DENEMO_ALT_OVERRIDE)
							(d-DirectivePut-note-display tag "^")
							(d-DirectivePut-note-ty tag -10)
							(if (< count number)
								(loop (1+ count))))))))					
	(d-RefreshDisplay)
	(d-SetSaved #f))	