;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;EndSecondTimeBar
(d-DirectivePut-standalone "Repeat")
(d-DirectivePut-standalone-minpixels "Repeat" 30)
(d-DirectivePut-standalone-postfix "Repeat" "
}}
	")
 (d-DirectivePut-standalone-graphic "Repeat" "EndSecondTimeBar")
 (d-DirectivePut-standalone-gy  "Repeat" -15)
 (d-DirectivePut-standalone-gy  "Repeat" -36)
(d-MoveCursorRight)
(d-RefreshDisplay)