;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;TitledPiece
         (MovementBookTitles::Do "TitledPiece" "titledPiece" "My Movement Title"  (_ "Give title for the movement or blank out to delete: "))
        