;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;BookArranger
        (BookTitles::Do "Arranger" "arranger" "My Arranger"  (_ "Give name for arranger etc or blank out to delete: "))
        