;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
(if (Music?)
 (ToggleDirective "chord" "postfix" "Congruentia" "^\\markup { \\musicglyph #\"scripts.usignumcongruentiae\"  }\n")
 #f)