;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;ForceBarline
(d-DirectivePut-standalone-minpixels "ForceBarline" 12)
(d-MoveCursorLeft)
(d-DirectivePut-standalone-postfix "ForceBarline" " \\bar \"|\"")
(d-DirectivePut-standalone-gx  "ForceBarline"  13)
(d-DirectivePut-standalone-gy  "ForceBarline"  0)
(d-DirectivePut-standalone-graphic "ForceBarline" "Barline")
(d-MoveCursorRight)
(d-RefreshDisplay)
