;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;Chapter
         (MovementBookTitles::Do "Chapter" "chapter" "My Heading"  (_ "Give heading or blank out to delete: "))
