;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;BookComposer
        (BookTitles::Do "Composer" "composer" "My Composer"  (_ "Give name for composer or blank out to delete: "))
