;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;SmallFontStart
(StandAloneDirectiveProto (cons "SmallFontStart" "% Start small notes		
		\\override NoteHead #'font-size = #-4
		\\override Accidental #'font-size = #-4
") #t "\nt")
