;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
(AttachDirective "staff" "prefix" "ContextGroupStaff" " \\new StaffGroup <<\n" DENEMO_OVERRIDE_GRAPHIC DENEMO_OVERRIDE_AFFIX)