;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
(d-DirectivePut-chord-postfix "SuggestedAccidental" "^\\markup \\natural")
(d-DirectivePut-chord-gy "SuggestedAccidental" -100)
(d-DirectivePut-chord-graphic "SuggestedAccidental" "
\xE2\x99\xAE
Sans
30")