;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;BookTitle
        (BookTitles::Do "Title" "title" "My Title"  (_ "Give a title for the whole score or blank out to delete: "))
        