;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;RestrainOpenStrings
(let ((tag "RestrainOpenStrings"))
	(StandAloneDirectiveProto (cons tag "\\set TabVoice.restrainOpenStrings = ##t ") #f "\n⨂\nDenemo\n30"))
