;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;PlaceFiguredBassBelow
(d-EditFiguredBass "$\\override Staff.BassFigureAlignmentPositioning #'direction = #DOWN $")