;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;;;;;; NormalBeamEndings
        ;;removes old syntax no longer valid for current LilyPond version
(d-ScoreBlockProlog "")
(d-SetSaved #f)

