;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;;ToggleFlageolet
(ChordAnnotation "ToggleFlageolet" "\\flageolet"    ToggleFlageolet::params    LG-Flageolet)
;(d-DirectivePut-chord-gy "Flageolet" 12)
