;;; Warning!!! This file is derived from those in actions/menus/... do not edit here

                     (StandAloneDirectiveProto (cons "TypesetOff" "
        	\\set Score.skipTypesetting = ##t       	
"))       
