;;; Warning!!! This file is derived from those in actions/menus/... do not edit here


(ChordComparison::ChordComparison (list ChordComparison::Major ChordComparison::Minor ChordComparison::Augmented ChordComparison::Diminished))

