;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
  ;;; ShowOssia
(d-Directive-standalone "StartStaff")
(d-DirectivePut-standalone-postfix "StartStaff" "\\startStaff ")
(d-DirectivePut-standalone-display "StartStaff" (_ "Show"))
(d-DirectivePut-standalone-ty "StartStaff" 40)
(d-RefreshDisplay)