;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
(if (d-MarkStatus) (d-DeleteSelectionLeaveEmpty)) ;; Goes to the beginning of the selection afterwards
(DenemoPaste #t)