;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
(if (Appending?)
	(d-MoveCursorLeft))
(d-MoveCursorLeft)
(d-AddDot)
(d-MoveCursorRight)
(d-Diminish)
(d-MoveCursorRight)
