;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
(Doublestroke #f
(cons "Double barline" d-DoubleBarline)  ;1
(cons "Repeat start" d-RepeatStart)  ;2
(cons "Repeat end" d-RepeatEnd)  ;3
(cons "Repeat end and start" d-RepeatEndStart)  ;4
(cons "First time bar open" d-OpenFirstTimeBar)  ;5
(cons "Second time bar open" d-OpenSecondTimeBar)  ;6
(cons "End first/second time section" d-EndVolta)  ;7
(cons "Closing barline" d-ClosingBarline)  ;8
(cons "Half-sized barline" d-HalfBarline)  ;9
(cons "Force normal barline" d-ForceBarline) ;0
)