;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
(d-PushPosition)
(d-UnsetMark)
(let loop ()
	(if (d-MoveToStaffUp)
		(loop)
		#t))
(let loop2 ()
	(if (d-StaffDown)
		(loop2)
		#t))			
(d-PopPosition)